function fn = makeFastSigmaMinFn(A,maxit,tol,n_fast)
% This is returns a function handle for computing the minimum singular
% value of A - zI via either svd (n < 55) or inverse Lanczos iteration.
% For details, see Chapter 39 of Spectra and Pseudospectra by Trefethen and
% Embree as well as psacore.m from EigTool.
%
%   Required inputs: 
%       A:          a matrix
%       maxit:      max number of inverse Lanczos iterations allowed
%       tol:        relative tol on smallest singular value
%       n_fast:     inverse iteration will be done for n >= n_fast
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: February 23, 2022
% License: AGPL
%
% =========================================================================
% |  makeFastSigmaMinFn.m                                                 |
% |  Copyright (C) 2022 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================
       
    if nargin < 2 || isempty(maxit)
        maxit   = 100;
    end
    if nargin < 3 || isempty(tol)
        tol     = 1.5*eps;
    end
    if nargin < 4 || isempty(n_fast)
        % EigTool decides uses plain SVD for n < 55 and Lanczos otherwise,
        % and even after many years, this still seems to be a very
        % appropriate dimension to use.  Though sometimes inverse Lanczos
        % is several times slower than svd at n=55, but this seems to go
        % away with repeated use, possibly to do just-in-time compilation?
        n_fast  = 55;
    end
    
    n = length(A);
    T = schur(A,'complex');
    I = speye(n);
    H = zeros(maxit + 1); % Preallocate H once so it doesn't change size.
    
    if n < n_fast
        fn = @directFn;
    else
        fn = @inverseLanczosFn;
    end
    
    function s_min = directFn(z)
        s_min = abs(min(svd(A - z*I)));
    end
    
    function s_min = inverseLanczosFn(z)
        % The code inside this function is essentially typed out from page
        % 375 of Spectra and Pseudospectra with some minor changes.
        % However, see the key comment below for an important correction!
        T1                  = T - z*I;
        T2                  = T1';
        [z_old,q_old,beta]  = deal(0);
        q                   = randn(n,1) + 1i*randn(n,1);
        q                   = q/norm(q);
        
        for j = 1:maxit
            v           = T1\(T2\q) - beta*q_old;
            alpha       = real(q'*v);
            v           = v - alpha*q;
            beta        = norm(v);
            q_old       = q;
            q           = v/beta;
            % H is preallocated once outside this function for speed
            H(j+1,j)    = beta;
            H(j,j+1)    = beta;
            H(j,j)      = alpha;
            z_max       = max(eig(H(1:j,1:j)));
            
            if abs(z_old/z_max - 1) < tol 
                break
            end
            z_old       = z_max;
        end
        % Page 375 of Spectra and Pseudospectra says ... = sqrt(sig), where
        % sig is z_max here, but this is a typo as we need also need to
        % take the reciprocal to go from sqrt of the largest eigenvalue of
        % B*B' to the smallest singular value of B!
        s_min   = 1/sqrt(z_max);
    end

end