function [fn0,fn1,fn2] = makeSigmaMinFn(A,E,polar,svd_type)
% This is returns a function handle for computing the minimum singular
% value of zE - A.
%
%   Required inputs: 
%       A:          a matrix
%       E:          a matrix
%       polar:      logical, use polar coordinates instead of Cartesian,
%                   i.e., re^(i*theta)
%       svd_type:   0: sparse mode, not yet implemented
%                   1: computes minumum singular value directly
%                   2: mathematically equivalent to 1 but does it by 
%                      computing maximum singular value of inverse of 
%                      zE - A (because svd can sometimes be very inaccurate 
%                      when the minimum singular value is needed).
%
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: July 14, 2019
% License: AGPL
%
% =========================================================================
% |  makeSigmaMinFn.m                                                     |
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================

    fn0 = @computeFunction;
    fn1 = @computeFirstOrder;
    fn2 = @computeSecondOrder;
    
    if nargin < 4
        svd_type = 1;
    end
    switch svd_type
        case 0
            getSigmaMin = @getSigmaMinSparse;
            fn2         = [];
        case 1
            getSigmaMin = @getSigmaMinDirect;
        case 2
            getSigmaMin = @getSigmaMinInverse;
    end
    
    n       = length(A);
    if isempty(E) || isEye(E)
        E   = speye(n);
    end
    getZ    = @(z) z*E - A;

    % For the partials of Z that are zero, it's faster to set them to a
    % scalar since this avoid matrix-vector products.  Furthermore, in 
    % Cartesian coordinates, all the partials of Z are constant so they
    % don't need to be updated; updating is only necessary in polar
    % coordinates.
    if nargin > 2 && polar
        toComplexScalar = @partialsPolar;
        updateZPartials = @updateZPartialsPolar;
        Z_dx    = [];
        Z_dy    = [];
        Z_dx2   = 0;
        Z_dy2   = [];
        Z_dxdy  = [];
    else
        toComplexScalar = @partialsCartesian;
        updateZPartials = @updateZPartialsCartesian;
        Z_dx    = E;
        Z_dy    = 1i*E;
        Z_dx2   = 0;
        Z_dy2   = 0;
        Z_dxdy  = 0;
    end
 
    % PRIVATE FUNCTIONS   
    
    % COMPUTE FUNCTION VALUE ONLY
    function [f,g,H,uo,vo] = computeFunction(x,y,partial)
                   
        z       = toComplexScalar(x,y);
        [f,u,v] = getSigmaMin(z);
        if nargin > 2 && (partial == 1 || partial == 2)
            g   = nan;
            H   = nan;
        else
            g   = nan(2,1);
            H   = nan(2);
        end
        uo      = u;
        vo      = v;
    end

    % COMPUTE GRADIENT ONLY   
    function [f,g,H,uo,vo] = computeFirstOrder(x,y,partial)     
   
        % For consistency with makeSigmaFn, we will first just compute the
        % minimum singular value and the first partial derivative(s).  Then
        % we will convert these all to be for the reciprocal of the minimum
        % singular value.
        [z,z_dx,z_dy]   = toComplexScalar(x,y);
        [f,u,v]         = getSigmaMin(z);
        t_dx            = u'*E*v;
           
        % The derivative of the singular value with respect to x (the
        % rectangular coordinate) can be converted to the derivative with
        % respect to y, its radius, or its angle by multiplying it by the
        % corresponding derivatives of z_dx or z_dy.
        if nargin < 3
            partial = 0;
        end
        switch partial
            case 1
                g   = z_dx * t_dx;
                H   = nan;
            case 2
                g   = z_dy * t_dx;
                H   = nan;
            otherwise
                g   = [z_dx; z_dy] * t_dx;
                H   = nan(2,1);
        end
        g       = real(g);
        uo      = u;
        vo      = v;
    end

    % COMPUTE FIRST AND SECOND DERIVATIVES
    function [f,g,H,uo,vo] = computeSecondOrder(x,y,partial)
        
        % For consistency with makeSigmaFn, we will first just compute the
        % minimum singular value and the first and second partial
        % derivative(s).  Then we will convert these all to be for the
        % reciprocal of the minimum singular value.
        [z,z_dx,z_dy,~,z_dy2,z_dxdy] = toComplexScalar(x,y);
        updateZPartials(z_dx,z_dy,z_dy2,z_dxdy);
        
        [f,u,v,Z,U,S,V] = getSigmaMin(z);
        t_dx            = u'*E*v;
        [UV,dUV]        = svdToEigs(U,S,V);
        partialFn       = secondPartialSingularValue(Z,UV,dUV);
        
        % The first and second derivatives Gp and Gpp of the matrix G with
        % respect to x (the rectangular coordinate) can be converted to the
        % corresponding first and second derivatives with respect to y
        % (either the rectangular coordinate or its radius) by multiplying
        % Gp and Gpp by the corresponding derivatives of z_dx or z_dy.  For
        % theta, Gp can be converted identically but converting Gpp also
        % requires a second term involving the converted Gp multiplied by
        % the second derivative of z with respect to theta.  The second
        % derivatives with respect to x and y can be similarly converted.  
        % Note that z_dx2 is zero in either rectangular or polar form.
        if nargin < 3
            partial = 0;
        end
        switch partial          
            case 1
                g       = z_dx * t_dx;  
                H       = partialFn(n,Z_dx,Z_dx2);
            case 2
                g       = z_dy * t_dx;
                H       = partialFn(n,Z_dy,Z_dy2);
            case 3
                g       = [z_dx; z_dy] * t_dx;
                f_dx2   = partialFn(n,Z_dx,Z_dx2);
                f_dy2   = partialFn(n,Z_dy,Z_dy2);
                H       = [ f_dx2; f_dy2 ];
            otherwise
                g       = [z_dx; z_dy] * t_dx;
                f_dx2   = partialFn(n,Z_dx,Z_dx2);
                f_dy2   = partialFn(n,Z_dy,Z_dy2);
                f_dxdy  = partialFn(n,Z_dx,Z_dy,Z_dxdy);
                H       = [f_dx2 f_dxdy; f_dxdy f_dy2];
        end
        g       = real(g);
        H       = real(H);
        uo      = u;
        vo      = v;
    end

    function [s,u,v,Z,U,S,V] = getSigmaMinSparse(z)
        error('large and sparse matrices not yet supported');
        Z           = getZ(z);
        [s,u,v]     = deal([]); % replace this with Propack
        [U,S,V]     = deal([]);
    end

    function [s,u,v,Z,U,S,V] = getSigmaMinDirect(z)
        Z           = getZ(z);
        [U,S,V]     = svd(Z);
        % svd can sometimes return singular values as -0, so we need to
        % take the absolute value of all them, to avoid any of their
        % reciprocals being -inf instead of +inf.
        S           = abs(S);
        
        s           = S(end);
        u           = U(:,end);
        v           = V(:,end); 
    end

    function [s,u,v,Z,U,S,V] = getSigmaMinInverse(z)
        Z           = getZ(z);
        [V,S,U]     = svd(inv(Z));
        % svd can sometimes return singular values as -0, so we need to
        % take the absolute value of all them, to avoid any of their
        % reciprocals being -inf instead of +inf.
        s           = abs(flip(abs(diag(S))));
        s(isnan(s)) = inf;
        S           = diag(1./s);
        U           = fliplr(U);
        V           = fliplr(V);
        
        s           = S(end);
        u           = U(:,end);
        v           = V(:,end); 
    end

    function updateZPartialsCartesian(varargin)
        % Z partials are fixed for Cartesian coordinates so do nothing
    end

    function updateZPartialsPolar(z_dr,z_dt,z_dt2,z_drdt)
        Z_dx    = z_dr*E;
        Z_dy    = z_dt*E;
        Z_dy2   = z_dt2*E;
        Z_dxdy  = z_drdt*E;
    end
end