function [data,h1,h2,h3] = getSepLamDemo(A,B,z0,z0s,use_line,use_sum,plots)
% This runs a prototype of our new method for computing sep_\lambda(A,B)^D,
% Algorithm 7.1, which returns the following information
%
%   data:   struct of the following info:
%   .sl     computed estimate of sep-lambda
%   .t      wall-clock time to run getSepLam
%   .n_f    total number of times f^D(z) was evaluated
%   .n_c    total number of certificates attempted
%   .n_pts  total number of angles d_eps(theta) was evaluated at, across
%           all certificates
%   .n_fin  total number of angles the final d_eps(theta) was evaluated at
%   .info   metadata associated with the computation returned by getSepLam
%   h1:     figure handle for pseudospectral plot
%   h2:     figure handle for d_eps components plot (linear scale)
%   h3:     figure handle for d_eps components plot (log scale)
% 
% and makes the following plots:
%   
%   a plot of the eigenvalues and final epsln-pseudospectra of A and B 
%   a plot of the final d_eps(theta) function (in linear scale)
%   a plot of the final d_eps(theta) function (in log10 scale).
%
% getSepLam can also approximate Varah's version of sep-lambda, as per
% Section 8.1.
%   
% This software is only intended as a proof-of-concept and is not yet tuned
% for efficiency.  Optimized routines for general computational use will be
% added to a future release of
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: November 7, 2019
% License: AGPL
%
% =========================================================================
% |  getSepLamDemo.m                                                      |
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================

    colors  = lines(3);
      
    opts.use_line       = use_line;
    opts.use_sum        = use_sum;
    opts.record         = true;
    opts.min_samples    = 17;
       
    t_start     = tic();
    [sl,info]   = getSepLam(A,B,z0,z0s,opts);
    t           = toc(t_start);

    % computed value of sep-lambda for A,B
    data.sl     = sl;
    % total running time:
    data.t      = t;
    % total # of f^D(z) or f^V(z) evals
    data.n_f    = info.n_fn;   
    % vector of interpolation pts per certificate
    t_evals     = info.test_evals;  
    % total # of certificates / restarts
    data.n_c    = length(t_evals);
    % total # of d_eps(theta) evals across all certificates
    data.n_pts  = sum(t_evals);
    % total # of d_eps(theta) evals for final certificates
    data.n_fin  = t_evals(end);
    % metadata from getSepLam
    data.info   = info;
    
    if nargin < 6 || ~plots
        [h1,h2,h3] = deal([]);
        return
    end

    eps1 = info.eps1;
    eps2 = info.eps2;
    
    cert = makeSepLamCertFn(A,B,1e-8,1e-8,use_line);
    cert.init(eps1,eps2,info.d_z0s(end));
    
    h1 = plotEigs(A,B);
    makePsPlots(A,B,eps1,eps2);
    largerTicks();
    tightFigure();
    zf = info.shared_z;
    % Optionally plot shared boundary point(s)
    % plot(complex(zf),'ko','LineWidth',2);

    z0 = info.d_z0s(end);
    az = angle(zf - z0);
    if use_line
        th_lb   = 0;
        ticksFn = @ticks0topi;
        if az < 0
            az  = az + pi;
        end
    else
        th_lb   = -pi;
        ticksFn = @tickspitopi;
    end

    g = info.d_eps;
    if isempty(g)
        g = chebfun(cert.fn,[th_lb,pi],'splitting','on');
    end
      
    h2 = figure();
    hold on 
    xlim([th_lb pi]);
    plot([th_lb pi],[0 0],'--','Color',[0.5 0.5 0.5]);
  
    plot(g,':','LineWidth',2,'Color',colors(1,:));
    plotCert(g);
    yr = ylim();
    yr(1) = -0.1*yr(2);
    ylim(yr);
    plot(az,0,'ro','LineWidth',2,'MarkerSize',8);
    ticksFn();
    largerTicks();
    tightFigure();

    h3 = figure();
    hold on
    xlim([th_lb pi]);
    plot(g,':','LineWidth',2,'Color',colors(1,:));
    plotCert(max(g,0));
    yr = ylim();
    yr(1) = 1e-8;
    ylim(yr);
    set(gca,'YScale','log');
    plot(az,1e-8,'ro','LineWidth',2,'MarkerSize',8);
    ticksFn();
    largerTicks();
    tightFigure();
    
    function plotCert(g)
       
%         % For debugging
%         function [d,c] = certFn(th)
%             [d,~,c] = cert.fn(th);
%         end
%         
%         ths = linspace(th_lb,pi,10000);
%         [d,c] = arrayfun(@certFn,ths);
%         
%         idx     = c == 0;
%         plot(ths(idx),d(idx),'.','Color',colors(2,:));  
%         idx     = c < 0;
%         plot(ths(idx),d(idx),'.','Color',colors(3,:));  
%         return
        
        pts = g.ends;
        mps = 0.5*(pts(1:end-1) + pts(2:end));
        
        for k = 1:length(mps)
            xs = linspace(pts(k),pts(k+1),1000);
            ys = g(xs);
            [~,~,dc] = cert.fn(mps(k));
            % take off the endpoints since for some reason, the chebfuns
            % can have the discontinuities as their end points
            
            dy  = abs(diff(ys));
            idx = 1:length(ys);
            if dy(1) > 0.1
                idx(1) = [];
            end
            if dy(end) > 0.1
                idx(end) = [];
            end
            if dc == 0
                plot(xs(idx),ys(idx),'-','LineWidth',2,'Color',colors(2,:));  
            end
            if dc < 0  % doesn't happen at sep-lambda
                plot(xs(idx),ys(idx),'--','LineWidth',2,'Color',colors(3,:));  
            end
        end  
        
    end

end