function [data,h1,h2] = getSepLamVDemo(A,B,plots)
% This runs a prototype of our new method, described in section 8.2 of the
% paper, for computing sep_\lambda(A,B)^V.  This routine returns the
% following information
%
%   data:   struct of the following info:
%   .sl     computed estimate of sep-lambda
%   .t      wall-clock time to run getSepLamV
%   .n_f    total number of times f^V(z) was evaluated
%   .n_v    total number of v(theta) was evaluated
%   h1:     figure handle for pseudospectral plot
% 
% and makes the following plots:
%   
%   a plot of the eigenvalues and final epsln-pseudospectra of A and B 
%
% This software is only intended as a proof-of-concept and is not yet tuned
% for efficiency.  Optimized routines for general computational use will be
% added to a future release of
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: November 7, 2019
% License: AGPL
%
% =========================================================================
% |  getSepLamVDemo.m                                                     |
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================

    t_start     = tic();
    [sl,info]   = getSepLamV(A,B);
    t           = toc(t_start);

    % computed value of sep-lambda for A,B
    data.sl     = sl;
    % total running time:
    data.t      = t;
    % total # of f^V(z) evals
    data.n_f    = info.n_fV;   
    % total # of v(theta) evaluations
    data.n_pts  = info.n_vtheta;
    % metadata from getSepLamV
    data.info   = info;
    
    if nargin < 3 || ~plots
        [h1,h2] = deal([]);
        return
    end

    eps1 = info.eps1;
    eps2 = info.eps2;
    
    h1 = plotEigs(A,B);
    makePsPlots(A,B,eps1,eps2);
    largerTicks();
    tightFigure();
    zf = info.shared_z;
    % Optionally plot shared boundary point(s)
    % plot(complex(zf),'ko','LineWidth',2);
      
    h2 = figure();
    hold on 
    plot(info.vtheta,'-','LineWidth',2);
    ticks0topi();
    largerTicks();
    tightFigure();

end