function [sl,info] = getSepLam(A,B,z0,z0s,options)
% This is a prototype implementation for computing sep_\lambda(A,B)^D, that
% is, Demmel's version of Sep Lambda, using the interpolation-based
% globality certificates proposed in the manuscript.  This implements
% Algorithm 7.1.
%
% By setting options.use_sum = true, it can also approximate Varah's
% version of sep-lambda.  This implements the extension of Algorithm 7.1 
% that is described in section 8.1. 
%
%   Required inputs: 
%       A:          a matrix 
%       B:          a matrix
%       z0:         search point or empty for default choice
%       z0s:        initial starting points in C
%       options:    optional struct of options:
%       .use_line       logical
%           use lines instead of rays, as per Section 7.6
%       .use_sum        logical
%           instead approximate Varah's sep-lambda, as per Section 8
%           NOTE: this prototype does not currently compute (8.2) but this
%                 should be done.
%       .record         logical
%           store metadata about computations and optimization trajectories
%       .min_samples    integer, power of two + one: 17, 33, 65, etc
%           setting for Chebfun's min_samples preference 
%
% See also getSepLamV. 
%   
% This software is only intended as a proof-of-concept and is not yet tuned
% for efficiency.  Optimized routines for general computational use will be
% added to a future release of
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: November 7, 2019
% License: AGPL
%
% =========================================================================
% |  getSepLam.m                                                          | 
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================

    % epsln will converge to the sep_\lambda^D(A,B)
    
    [z0_df,getNewZ0,dA,dB,shared] = checkEigs(A,B,0.05);
    
    if isempty(z0_df)
        % A and B have a matching eigenvalue so answer is 0
        sl              = 0;
        info.iters      = 0;
        info.d_eps      = [];
        info.d_z0s      = [];
        info.shared_z   = shared;
        info.eps1       = 0;
        info.eps2       = 0;
        info.recs       = {};
        info.test_evals = 0;
        info.test_avgs  = 0;
        info.test_times = 0;
        info.t_test     = 0;
        info.t_opt      = 0;
        info.n_test     = 0;
        info.n_opt      = 0;
        info.n_fn       = 0;
        info.t_par      = 0;
        info.test_avg   = 0;
        return
    end
    if isempty(z0)
        z0          = z0_df;
    end
    
    if nargin < 4
        use_line    = true;
        use_sum     = false;
        record      = false;
        n_samp      = 17;
    else
        use_line    = options.use_line;
        use_sum     = options.use_sum;
        record      = options.record;
        n_samp      = options.min_samples;   % 17, 33, 65
    end
     
    tol         = 1e-14;
    restart_tol = 1e-6;
    maxit       = 100;
    
    cert_opts.use_line      = use_line;
    cert_opts.imag_tol      = 1e-8;
    cert_opts.eps_tol       = 1e-8;
    cert_opts.min_samples   = n_samp;
      
    th_range    = [-pi,pi];  
    if use_line
        th_range(1) = 0;
    end
        
%     is_sym      = (isRealValued(A) && isRealValued(B));
%    
%     if is_sym
%         idx         = imag(z0s) < 0;
%         z0s(idx)    = conj(z0s(idx));
%         z0s         = unique(z0s);
%         th_range(1) = 0;  
%     end
        
    t_opt   = 0;
    n_test  = 0;
    n_opt   = 0;
    n_fn    = 0;
    sl      = inf;
    eps1    = inf;
    eps2    = inf;
    
    if use_sum
        [sl,eps1,eps2,shared] = varahTrivialUB(A,B,dA,dB);
    end
    
    t_tests     = [];
    test_evals  = [];
    test_avgs   = [];
    d_z0s       = [];
   
    sigFn       = getSigmaMinAB(A,B,use_sum,1);
    findMin     = @(z) findMinimizer(sigFn,z,record);
    rec         = findBestMin(z0s);
   
    cert        = makeInterpCert(A,B,z0,getNewZ0,th_range,cert_opts);
    recs        = { rec };
  
    for iters = 1:maxit
        % Do globality certificate computations 
        t2 = tic();
        [is_global,n_evals,avg,dfn,d_z0] = cert.isGlobal(eps1,eps2);
        t_test      = toc(t2);
        t_tests     = [t_tests t_test]; 
        test_evals  = [test_evals n_evals];
        test_avgs   = [test_avgs avg];
        d_z0s       = [d_z0s d_z0];
        
        z           = cert.getPoints();
        target      = sl*(1 - restart_tol);
        sl_prev     = sl;
        rec         = findBestMin(z,target);
        recs        = [ recs rec ];
        rel_diff    = abs((sl_prev - sl)/sl);
        % If relative change was small, consider converged
        if rel_diff <= tol
            break
        end
    end
    
    plot_interp = false;
    if plot_interp
        figure
        if isempty(dfn)
            dfn     = cert.getInterp(eps1,eps2);
        end
        plot(dfn,'LineWidth',2);
    end
    
    info.iters      = iters;
    info.d_eps      = dfn;
    info.d_z0s      = d_z0s;
    info.shared_z   = shared;
    info.eps1       = eps1;
    info.eps2       = eps2;
    info.recs       = recs;
    info.test_evals = test_evals;
    info.test_avgs  = test_avgs;
    info.test_times = t_tests;
    info.t_test     = sum(t_tests);
    info.t_opt      = t_opt;
    info.n_test     = n_test;
    info.n_opt      = n_opt;
    info.n_fn       = n_fn;
    
    [t_par,~,test_avg]  = cert.getData();
    info.t_par          = t_par;
    info.test_avg       = test_avg;
      
    % Private functions
    
    function rec_opt = findBestMin(zs,target)
        rec_opt     = [];
        t1          = tic;
        for j = 1:length(zs)
            [zf,f,e1,e2,r] = findMin(zs(j));
            n_opt   = n_opt + 1;
            n_fn    = n_fn + r.n_fn;
            if f < sl
                sl      = f;
                eps1    = e1;
                eps2    = e2;
                shared  = zf;
                rec_opt = r;
            end
            if nargin > 1 && sl < target
                break
            end
        end
        t_opt = t_opt + toc(t1);
    end

end