function [sl,info] = getSepLamV(A,B)
% This is a prototype implementation for computing sep_\lambda(A,B)^V, that
% is, Varah's version of Sep Lambda.  This implements the method described
% in section 8.2 of the manuscript. 
%
%   Required inputs: 
%       A:          a matrix 
%       B:          a matrix
%
% See also getSepLam. 
%   
% This software is only intended as a proof-of-concept and is not yet tuned
% for efficiency.  Optimized routines for general computational use will be
% added to a future release of
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: May 10, 2021
% License: AGPL
%
% =========================================================================
% |  getSepLamV.m                                                         | 
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================


    [sl,eps1,eps2]      = deal(0);
    [z0,~,dA,dB,shared] = checkEigs(A,B,0.05);
    [n_fV,n_vtheta]     = deal(0);
            
    if ~isempty(shared)
        info        = getInfo();        
        return
    end

    maxit           = [];
    tol             = [];
    n_fast          = [];
    fA              = makeFastSigmaMinFn(A,maxit,tol,n_fast);
    fB              = makeFastSigmaMinFn(B,maxit,tol,n_fast);
    eps0            = fA(z0) + fB(z0);
    sl              = eps0;
    
    % Run Chebfun on on v(theta) to find a minimizer
    g_vtheta        = chebfun(@getLineMin,[0 pi],'splitting','on');
    [~,theta]       = min(g_vtheta);
    [~,r]           = getLineMin(theta);
    fnPolar(r,theta);
    
    % Refine minimizer computed by Chebfun via local optimization
    fn              = getSigmaMinAB(A,B,true,1);
    z_cheb          = z0 + r*exp(1i*theta);
    findMinimizer(@fnOpt,z_cheb);
    
    % Check the trivial upper bound as this is sometimes the right answer
    [ub,eps1_ub,eps2_ub,shared_ub] = varahTrivialUB(fA,fB,dA,dB);
    
    if ub < sl
        sl          = ub;
        eps1        = eps1_ub;
        eps2        = eps2_ub;
        shared      = shared_ub;
    end
    
    info            = getInfo(g_vtheta);

    function info = getInfo(g_vtheta)
        if nargin < 1
            g_vtheta    = [];
        end
        info.vtheta     = g_vtheta;
        info.z0         = z0;
        info.shared_z   = shared;
        info.eps1       = eps1;
        info.eps2       = eps2;
        info.n_fV       = n_fV;
        info.n_vtheta   = n_vtheta;
    end

    function [f,g,e1,e2] = fnOpt(x,y)
        [f,g,e1,e2] = fn(x,y);
        if f < sl
            sl      = f;
            eps1    = e1;
            eps2    = e2;
            shared  = x + 1i*y;
        end
        n_fV        = n_fV + 1;
    end
    
    function f = fnPolar(r,theta)
        zp  = z0 + r*exp(1i*theta);
        e1  = fA(zp);
        e2  = fB(zp);
        f   = e1 + e2;
        if f < sl
            sl      = f;
            eps1    = e1;
            eps2    = e2;
            shared  = zp;
        end
        n_fV        = n_fV + 1;
    end

    function [y,r] = getLineMin(theta)
        [r1,r2] = bracketLevelSetAB(A,B,eps0,z0,theta);
        if isempty(r1) || r1 > 0
            r1  = 0;
        end
        if isempty(r2) || r2 < 0
            r2  = 0;
        end
        gline   = chebfun(@(r) fnPolar(r,theta),[r1 r2],'splitting','on');
        [y,r]   = min(gline);
        y       = fnPolar(r,theta);
        n_vtheta = n_vtheta + 1;
    end

end

function [r1,r2] = bracketLevelSetAB(A,B,epsln,z0,theta)
    [A1,A2] = bracketLevelSet(A,epsln,z0,theta);
    [B1,B2] = bracketLevelSet(B,epsln,z0,theta);
    r1      = max([A1 B1]);
    r2      = min([A2 B2]);
end

function [r1,r2] = bracketLevelSet(A,epsln,z0,theta)
    tol     = 1e-8;
    C_theta = ps1dMatrix(A,epsln,z0,theta);
    [~,~,r] = getEigenvalues(C_theta,tol);
    r1      = min(r);
    r2      = max(r);
end

function [z,zl,r] = getEigenvalues(C,imag_tol)
    z       = eig(C);

    % Find imaginary eigenvalues and get their imaginary parts 
    idx     = abs(real(z)) <= imag_tol;
    z_imag  = z(idx);
    r       = imag(z_imag);
    
    % Get all eigenvalues in the closed left half plane, using the
    % (nearly) imaginary eigenvalues above set to be exactly imaginary
    zl      = z;
    zl(idx) = 1i*r;
    idx     = real(zl) <= 0;
    zl      = zl(idx);
end