function cert = makeInterpCert(A,B,z0,getNewZ0,range,opts)
% This builds an object for computing the interpolation-based globality
% certificates described in sections 4,5,6,8 of the submitted manuscripted.
% For Demmel's sep-lambda,arguments eps1 and eps2 below must be equal.
%
%   INPUT:
%   A:              an m by m matrix
%   B:              an n by n matrix
%   z0:             search point for the certificate
%   getNewZ0:       fn handle to get new z0 if current one isn't compatible
%   range:          1 by 2 vector specifying the domain of the certificate
%                   distance function 
%   opts:           struct of options
%   .eps_tol        relative difference for how close a singular value of 
%                   A - z0I or A - z0I can be to the current epsilon
%                   values.  If this violated, a new z0 will automatically
%                   be chosen.
%   .imag_tol       when computing the distance function value, an 
%                   eigenvalue z will be considered imaginary  if 
%                   abs(real(z)) <= imag_tol holds.
%   .use_line       do searches via lines rather than just a ray in a
%                   single direction 
%   .min_samples    value for Chebfun preference
%
%   OUTPUT:
%   cert: a struct of function handles 
%
%   [a,b,l,dAB] = cert.getComp(eps1,eps2,thetas) 
%       Evaluates the different angles thetas are returns the values of
%       the constituent functions at these points:
%           a:   a_eps(theta) defined in (4.5a)
%           b:   b_eps(theta), the analogue of a_eps(theta) for matrix B
%           l:   l_eps(theta) defined in (5.1a) 
%           dAB: d_eps^{AB}(theta) defined in (6.1a)
%
%   [g,n_evals,avg,z0_used] = cert.getInterp(eps1,eps2) 
%       Builds a chebfun g of d_eps(theta) for the given eps1, eps2, and
%       z0.  Returns chebfun g, the number of function evaluations, the
%       average number of points provided by Chebfun at a time, and the
%       search point z0 used.
%
%   [tf,n_evals,avg,g,z0_used] = cert.isGlobal(eps1,eps2,can_restart)
%       Returns true if d_eps(theta) < 0 is never encountered when
%       building an interpolant of d_eps(theta).  If can_restart=true, the
%       method will terminate as soon as any angle making d_eps(theta) < 0
%       is encountered and the chebfun g will not built.  can_restart can
%       also be set as a functiom to specify custom termination criteria.
%
%   [z] = cert.getPoints()
%       Returns the points, if any, found in both eps1-pseudospectrum of A
%       and the eps2-pseudospectrum of B.  Such points are associated with
%       angles where d_eps(theta) < 0. The points z are given as complex
%       scalars.
% 
%   [t_p,n_evals,n_avg] = cert.getData()
%       Returns meta data about the computations so far.
%       t_p:        total time spent in the main (par)loop of computing the
%                   distance function for a vector of different angles
%       n_evals:    total number of times the distance function has been
%                   evaluated 
%       n_avg:      average vector length of angles to evaluate the
%                   distance function
% 
% See also makeSepLamCertFn.
%
% 
% This software is only intended as a proof-of-concept and is tuned for 
% demonstration purposes, not efficiency.  Optimized routines for 
% general computational use will be added to a future release of 
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%   
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: September 16, 2019
% License: AGPL
%
% =========================================================================
% |  makeInterpCert.m                                                     |
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================
    
    imag_tol    = opts.imag_tol;
    eps_tol     = opts.eps_tol;
    use_line    = opts.use_line;
    ms          = opts.min_samples;
    cf_args     = {range,'splitting','on','novectorcheck','minSamples',ms};
             
    obj         = makeSepLamCertFn(A,B,imag_tol,eps_tol,use_line);
    initFn      = obj.init;
    certFn      = obj.fn;
    compFn      = obj.compFn;
    
    z_cur       = [];
    
    t_par       = 0;
    n_pts       = 0;
    n_calls     = 0;
       
    cert.getInterp   = @getInterp;
    cert.isGlobal    = @isGlobal;
    cert.getPoints   = @getPoints;
    cert.getData     = @getData;
    cert.getComp     = @getComponents;
    
    function [a,b,l,d] = getComponents(eps1,eps2,thetas)
        initialize(eps1,eps2);
        [a,b,l,d] = arrayfun(compFn,thetas);
    end
   
    function [g,n_evals,avg,z0_used] = getInterp(eps1,eps2)
        evals   = n_pts;
        n_dist  = n_calls;
        initialize(eps1,eps2);
        fn      = @(theta) evalLevelFn(theta);
        g       = chebfun(fn,cf_args{:});
        n_evals = n_pts - evals;
        avg     = n_evals/(n_calls - n_dist);
        z0_used = z0;
    end

    function [tf,n_evals,avg,g,z0_used] = isGlobal(eps1,eps2,can_restart)
 
        tf      = true;     % assume gamma is globally <= min value
        evals   = n_pts;
        n_dist  = n_calls;
        z_cur   = [];
        g       = [];
        
        if nargin > 2 && isa(can_restart,'function_handle')
            rFn = can_restart;
        else
            rFn = @(varargin) true;
        end  
        
        initialize(eps1,eps2);
        z0_used = z0;
        fn      = @(theta) evalLevelFn(theta,rFn);
        
        try 
            g   = chebfun(fn,cf_args{:});
        catch err
            id  = err.identifier;
            switch id
                case 'interpCertify:canRestart'
                    tf      = false;
                    n_evals = n_pts - evals;
                    avg     = n_evals/(n_calls - n_dist);
                    return
                otherwise
                    err.rethrow();
            end
        end
  
        % d_eps is nonnegative at all interpolated points but we can't rule
        % out that the interpolant is negative between interpolated points.
        % Thus we check at the minimizer(s):
        [~,th]  = min(g);
       
        try 
            fn(th);
        catch err
            id  = err.identifier;
            switch id
                case 'interpCertify:canRestart'
                    tf      = false;
                    n_evals = n_pts - evals;
                    avg     = n_evals/(n_calls - n_dist);
                    return
                otherwise
                    err.rethrow();
            end
        end
        % The level-set function appears nonnegative at the global 
        % minimizer(s)ositive at all midpoints so we can assume that 
        % the interior of the two pseudospectra do not intersect.
        n_evals = n_pts - evals;
        avg     = n_evals/(n_calls - n_dist);
    end

    function z = getPoints()
        z       = z_cur;
    end

    function [t_p,n_evals,n_avg] = getData()
        t_p     = t_par;
        n_evals = n_pts;
        n_avg   = n_pts/n_calls;
    end

    % Private function
    
    function d = evalLevelFn(thetas,canRestartFn)
        n   = length(thetas);
        d   = zeros(size(thetas));
        d_z = false(1,n);
        z   = cell(1,n);
        
        t_start     = tic();
        for j = 1:n
            theta       = thetas(j);
            [d(j),zn]   = certFn(theta);
            d_z(j)      = d(j) < 0;
            if ~isempty(zn)
                z{j}    = zn(:);
            end
        end
        t_dur           = toc(t_start);
        t_par           = t_par + t_dur;
        n_pts           = n_pts + n;
        n_calls         = n_calls + 1;

        if nargin < 2
            return
        end
        % Concatenate all points corresponding to overlap areas
        z_cur = cat(1,z{:});
        if any(d_z) && canRestartFn(z_cur)
            error('interpCertify:canRestart','');
        end
    end

    function initialize(eps1,eps2)  
        while true 
            assert(isscalar(z0),'search point must be a scalar');
            try 
                initFn(eps1,eps2,z0);
                return
            catch
                z0 = getNewZ0();
            end
        end
    end

end
