function obj = makeSepLamCertFn(A,B,imag_tol,eps_tol,use_line)
% This builds an object for computing the interpolation-based globality
% certificates described in sections 4,5,6,8 of the submitted manuscripted.
% For Demmel's sep-lambda,arguments eps1 and eps2 below must be equal.
%
%   INPUT:
%   A:              an m by m matrix
%   B:              an n by n matrix
%   z0:             search point for the certificate
%   getNewZ0:       fn handle to get new z0 if current one isn't compatible
%   range:          1 by 2 vector specifying the domain of the certificate
%                   distance function 
%   opts:           struct of options
%   .eps_tol        relative difference for how close a singular value of 
%                   A - z0I or A - z0I can be to the current epsilon
%                   values.  If this violated, a new z0 will automatically
%                   be chosen.
%   .imag_tol       when computing the distance function value, an 
%                   eigenvalue z will be considered imaginary  if 
%                   abs(real(z)) <= imag_tol holds.
%   .use_line       do searches via lines rather than just a ray in a
%                   single direction 
%   .min_samples    value for Chebfun preference
%
%   OUTPUT:
%   obj: a struct of function handles 
%
%   init(eps1,eps2,z0)
%       Checks that the values of eps1 and eps2 are compatible with search
%       points.  If not, an error will be throw. This must be called at
%       least once before calling either of the other two functions.
%
%   [d,zAB,d_case] = certFn(theta)
%       For the given values of eps1, eps2, this evaluates d_eps(theta)
%       defined in (6.2) or when eps1,eps2 are not equal, (8.1).  It
%       returns:
%           d:      the value of d_eps(theta)
%           zAB:    detected points (if any) of the intersection of the
%                   eps1-pseudospectrum of A and eps2-pseudospectrum of B
%           d_case: 1   if d_eps(theta) = a_eps(theta) + b_eps(theta)
%                   0   if d_eps(theta) = d_eps^{AB}(theta)
%                   -1  if d_eps(theta) = l_eps(theta).
%
%   [a,b,l,dAB] = getComp(theta) 
%       Evaluates the different components of d_eps(theta) and returns
%       their respective values separate
%           a:   a_eps(theta) defined in (4.5a)
%           b:   b_eps(theta), the analogue of a_eps(theta) for matrix B
%           l:   l_eps(theta) defined in (5.1a) 
%           dAB: d_eps^{AB}(theta) defined in (6.1a)
%
% 
% This software is only intended as a proof-of-concept and is tuned for 
% demonstration purposes, not efficiency.  Optimized routines for 
% general computational use will be added to a future release of 
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%   
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: September 16, 2019
% License: AGPL
%
% =========================================================================
% |  makeSepLamCertFn.m                                                   |
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================

    
    % Smoothing parameters
    k1  = 2;
    k2  = 4;
    c1  = 0.1;
    c2  = 0.4;
    
    n_fast      = 55; % Dimension to start using inverse Lanczos
    sminA       = makeFastSigmaMinFn(A,[],[],n_fast);
    sminB       = makeFastSigmaMinFn(B,[],[],n_fast);
    checkEpsFn  = getCheckEpsilonValuesFn(A,B,eps_tol);
    
    %imag_tol = 1e-12;
    
    z0      = [];
    eps1    = [];
    eps2    = [];
    fA      = [];
    fB      = [];
    
    obj.init    = @initialize;
    obj.fn      = @certFn;
    obj.compFn  = @getComponents;
    
    function initialize(epsln1,epsln2,z0_in)
        checkEpsFn(epsln1,epsln2,z0_in);
        z0      = z0_in;
        eps1    = epsln1;
        eps2    = epsln2;
        fA      = @(r,theta) sminA(z0 + r*exp(1i*theta));
        fB      = @(r,theta) sminB(z0 + r*exp(1i*theta));
    end

    function [a,b,l,dAB] = getComponents(theta)
         
        theta   = wrapToPi(theta);

        C_theta         = ps1dMatrix(A,eps1,z0,theta);
        S_theta         = ps1dMatrix(B,eps2,z0,theta);  
        [za,a_left,ra]  = getEigenvalues(C_theta,imag_tol);
        [zb,b_left,rb]  = getEigenvalues(S_theta,imag_tol);

        if use_line
            % Unlike certFn, we don't care about efficiency here (since
            % this is just for making plots) so keep the code simple
            a   = min([pseudoFn(a_left,k1) pseudoBackFn(a_left,k1)]);
            b   = min([pseudoFn(b_left,k1) pseudoBackFn(b_left,k1)]);
            tA  = getIntervalsLine(fA,eps1,theta,ra);
            tB  = getIntervalsLine(fB,eps2,theta,rb);    
        else
            a   = pseudoFn(a_left,k1);
            b   = pseudoFn(b_left,k1);
            tA  = getIntervalsRay(fA,eps1,theta,ra);
            tB  = getIntervalsRay(fB,eps2,theta,rb);
        end
        lp      = getOverlap(tA,tB);
        l       = -lp;
        dAB     = min([ getDist(fA,eps1,theta,tA,rb,use_line) 
                        getDist(fB,eps2,theta,tB,ra,use_line)] );
    end
    
    function [d,zAB,d_case] = certFn(theta)

        zAB     = [];
        d_case  = 1;
        theta   = wrapToPi(theta);

        C               = ps1dMatrix(A,eps1,z0,theta);
        S               = ps1dMatrix(B,eps2,z0,theta);  
        [za,a_left,ra]  = getEigenvalues(C,imag_tol);
        [zb,b_left,rb]  = getEigenvalues(S,imag_tol);

        if use_line
            dp  = pseudoFn(a_left,k1) + pseudoFn(b_left,k1);
            dn  = pseudoBackFn(a_left,k1) + pseudoBackFn(b_left,k1);
            d   = min(dp,dn);
            if d > 0
                return
            end
            if max(dp,dn) == 0
                tA  = getIntervalsLine(fA,eps1,theta,ra);
                tB  = getIntervalsLine(fB,eps2,theta,rb);
            elseif dp == 0
                tA  = getIntervalsRay(fA,eps1,theta,ra);
                tB  = getIntervalsRay(fB,eps2,theta,rb);
            else
                tA  = getIntervalsRayBack(fA,eps1,theta,ra);
                tB  = getIntervalsRayBack(fB,eps2,theta,rb);
            end
        else
            d   = pseudoFn(a_left,k1) + pseudoFn(b_left,k1);
            if d > 0
                return
            end
            tA  = getIntervalsRay(fA,eps1,theta,ra);
            tB  = getIntervalsRay(fB,eps2,theta,rb);
        end

        [d,tAB] = getOverlap(tA,tB);
        d       = -d;
        zAB     = z0 + tAB*exp(1i*theta);
        if d < 0
            d_case = -1;
            return
        end
        d_case  = 0;
        zAB     = [];
%         d       = getLength(tA,tB,k1,c1)^2;
%         d       = getMinSep(tA,tB);
%         d       = getDist(fA,eps1,theta,tA,rb,use_line);
%         d       = getDist(fB,eps2,theta,tB,ra,use_line);
        d       = min([ getDist(fA,eps1,theta,tA,rb,use_line) 
                        getDist(fB,eps2,theta,tB,ra,use_line)] );
%         d  =   getDist(fA,eps1,theta,tA,rb,use_line) + ...
%                         getDist(fB,eps2,theta,tB,ra,use_line);
%         d = powSmooth(d,k1,c1);
    end
end

function s = getMinSep(tA,tB)
    tA  = tA(:);
    tB  = tB(:);  
    if isempty(tA) || isempty(tB)
        s = 0;
        return
    end
    s   = min(arrayfun(@(t) min(abs(tB - t)),tA));
end

function l = getLength(tA,tB,k,c)
    if isempty(tA)
        lA = 0;
    else
        lA = sum(powSmooth(abs(tA(:,2) - tA(:,1)),k,c));
    end
    if isempty(tB)
        lB = 0;
    else
        lB = sum(powSmooth(abs(tB(:,2) - tB(:,1)),k,c));
        %lB = sum(abs(tB(:,2) - tB(:,1)));
        %lB = powSmooth(lB,k,c);
    end
    l = min(lA,lB);
end

function [d,c] = getDist(fA,epsln,theta,tA,r,use_line)
    
    r       = unique(r);
    if ~use_line
        r   = r(r > 0);
    end
    if isempty(tA) || isempty(r)
        d       = 0;
        c       = [];
        return
    end
    
    nT      = size(tA,1); 
    % This relies on the intervals being sorted in order
    for j = 1:nT
        t       = tA(j,:);
        % points strictly inside the interval
        idx     = arrayfun(@(r) t(1) < r && r < t(2),r);
        r(idx)  = nan;
        % points at an endpoint of the interval
        idx     = arrayfun(@(r) t(1) == r || r == t(2),r);
        r(idx)  = inf;
    end
    
    % Get rid of the points inside the intervals of tA
    idx     = isnan(r);
    r(idx)  = [];
    
    % If there a point is exactly at an endpoint, distance is zero
    idx     = isinf(r);
    if any(idx)
        d   = 0;
        c   = r(idx);
        return
    end
    r(idx)  = [];
    
    % Only points outside the intervals remain so do singular value
    % evaluations
    % Account for rounding error that points may return negative values
    fn      = @(r) max([fA(r,theta) - epsln, 0]);
    el      = arrayfun(fn,r);
    [d,idx] = min(el);
    c       = r(idx);
    
    if isempty(d)
        d   = 0;
        c   = [];
    end
end

function [oAB,tAB] = getOverlap(tA,tB)
   
    nA  = size(tA,1);
    nB  = size(tB,1);
    j   = 1;
    k   = 1;
    
    oAB = 0;
    tAB = zeros(nA + nB - 1,2);
    l   = 0;
    
    while j <= nA && k <= nB
        a1      = tA(j,1);
        a2      = tA(j,2);
        % Find first interval of B whose end point is equal or to the right
        % of the current interval of A
        while k <= nB && tB(k,2) < a1  
            k   = k + 1;
        end
        if k > nB
            break
        end
        b1      = tB(k,1);
        b2      = tB(k,2);
        if a2 < b1 
            % no ovelap
            j   = j + 1;
        else
            % overlap (though possibly just at at endpoint)
            o1  = max([a1 b1]);
            o2  = min([a2 b2]);
            oAB = oAB + (o2 - o1);
            l   = l + 1;
            tAB(l,:) = [o1 o2];
            if a2 <= b2
                % Interval from tA has no more overlap
                j = j + 1;
            else
                % Interval from tB has no more overlap
                k = k + 1;
            end
        end
    end
    tAB     = tAB(1:l,:);
end

function t = getIntervalsLine(fA,epsln,theta,r)
    % For lines, make sure the r values are in increasing order and unique
    r       = unique(r);
    t       = getIntervals(fA,epsln,theta,r);
end

function t = getIntervalsRay(fA,epsln,theta,r)
    % For rays, make sure the r values are in increasing order, unique,
    % and if any r <= 0, get rid of all but the largest one 
    r       = unique(r);
    n_start = max(1,nnz(r <= 0));
    r       = r(n_start:end);
    %np      = nnz(r <= 0);
    %idx     = 1:(np-1);
    %r(idx)  = [];
    t       = getIntervals(fA,epsln,theta,r);
    % If the first interval crosses 0, get rid of the negative part
    if ~isempty(t) && t(1) < 0
        t(1) = 0;
    end
end

function t = getIntervalsRayBack(fA,epsln,theta,r)
    % For rays, make sure the r values are in increasing order, unique,
    % and if any r >= 0, get rid of all but the smallest one
    r       = unique(r);
    n_end   = min(nnz(r < 0) + 1,length(r));
    r       = r(1:n_end);
    t       = getIntervals(fA,epsln,theta,r);
    % If the last interval crosses 0, get rid of the positive part
    if ~isempty(t) && t(end) > 0
        t(end) = 0;
    end
end

function t = getIntervals(fA,epsln,theta,r)
    % r must be in sorted order
    if length(r) < 2
        t   = zeros(0,2);
        return
    end
    
    % We will determine which intervals are in the epsln-pseudospectrum of
    % A by checking the value at the midpoints of consecutive points in r
    fn      = @(r) fA(r,theta) <= epsln;
    mp      = r(1:end-1) + 0.5*diff(r);
    idx     = find(arrayfun(fn,mp));
    t       = [r(idx) r(idx+1)];
end

function [z,zl,r] = getEigenvalues(C,imag_tol)
 
    z   = eig(C);

    % Find imaginary eigenvalues and get their imaginary parts 
    idx     = abs(real(z)) <= imag_tol;
    z_imag  = z(idx);
    r       = imag(z_imag);
    
    % Get all eigenvalues in the closed left half plane, using the
    % (nearly) imaginary eigenvalues above set to be exactly imaginary
    zl      = z;
    zl(idx) = 1i*r;
    idx     = real(zl) <= 0;
    zl      = zl(idx);
end

function d = pseudoFn(z,k)
    d   = min(abs(angle(-1i*z)));
    d   = d^k;
end

function d = pseudoBackFn(z,k)
    z   = real(z) - 1i*imag(z);
    d   = pseudoFn(z,k);
end

