function [z_opt,f_opt,eps1_opt,eps2_opt,rec] = findMinimizer(fn,z0,record)
% Given a function handle fn for evaluating [f,g] = f(x,y), the value and
% gradient of some function in two real variables, this finds routine runs
% GRANSO to find a minimizer of it, though it may also converge to a point
% which is stationary.
%
%   Required inputs: 
%       fn:         fn to minimizer 
%       z0:         Initial point given as a complex scalar
%       record:     logical: record iterates
%
% This software is only intended as a proof-of-concept and is tuned for 
% demonstration purposes, not efficiency.  Optimized routines for 
% general computational use will be added to a future release of 
%
% ROSTAPACK: RObust STAbility PACKage
% http://www.timmitchell.com/software/ROSTAPACK/
%   
%
% Authored by: Tim Mitchell
% Email: mitchell@mpi-magdeburg.mpg.de
% Date: November 7, 2019
% License: AGPL
%
% =========================================================================
% |  findMinimizer.m                                                      |
% |  Copyright (C) 2019 Tim Mitchell                                      |
% |                                                                       |
% |  This program is free software: you can redistribute it and/or modify |
% |  it under the terms of the GNU Affero General Public License as       |
% |  published by the Free Software Foundation, either version 3 of       |
% |  the License, or (at your option) any later version.                  |
% |                                                                       |
% |  This program is distributed in the hope that it will be useful,      |
% |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
% |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
% |  GNU Affero General Public License for more details.                  |
% |                                                                       |
% |  You should have received a copy of the GNU Affero General Public     |
% |  License along with this program.  If not, see                        |
% |  <http://www.gnu.org/licenses/agpl.html>.                             |
% =========================================================================

    n_fn        = 0;
    record      = nargout > 2 && nargin > 2 && record;
 
    maxit       = 1000;
    
    z_opt       = 0;
    f_opt       = inf;
    eps1_opt    = inf;
    eps2_opt    = inf;
    
    if record
        [halt_log_fn, get_log_fn]   = makeHaltLogFunctions(maxit);
    else
        halt_log_fn = [];
    end

    opts.x0                     = [real(z0); imag(z0)];
    opts.maxit                  = maxit;  
    opts.opt_tol                = 1e-14;
    opts.print_level            = 0;
    opts.halt_log_fn            = halt_log_fn;
    opts.quadprog_info_msg      = false;
    opts.prescaling_info_msg    = false;
    
    % Run optimization; we handle keeping track of the best answer in objFn
    granso(2,@objFn,[],[],opts);
 
    if record
        rec         = get_log_fn();
        rec.n_fn    = n_fn;
    end
    
    function [f,g] = objFn(x)
        [x,y]           = deal(x(1),x(2));
        [f,g,e1,e2]     = fn(x,y);
        n_fn            = n_fn + 1;
        
        if f < f_opt
            f_opt      = f;
            eps1_opt   = e1;
            eps2_opt   = e2;
            z_opt      = x + 1i*y;
        end
    end

end